find_pokemon_stats:	; hl=pokemon id ; output : hl=pokemon stats
	ld a,(hl)

find_pokemon_stats_a:
	or a
	ret z
	ld hl,pokemon_stats
	ld bc,pokemon_stats_lenght-pokemon_stats

find_pokemon_stats_loop:
	add hl,bc
	dec a
	jp nz,find_pokemon_stats_loop
	or a
	sbc hl,bc
	ret

find_pokemon_data:	; input : hl=pokemon id ; output : hl=pokemon data
	ld a,(hl)
	ld b,a
	ld hl,pokemon_index

find_pokemon_data_loop:
	inc hl
	inc hl
	djnz find_pokemon_data_loop
	dec hl
	dec hl
	bcall _ldhlind
	ret

find_pokemon_attack:	; input : hl=attack id ; output : hl=attack data
	ld a,(hl)

find_pokemon_attack_a:
	ld b,a
	ld hl,attack_index
	jp find_pokemon_data_loop

find_type:
	ld a,(hl)
	ld b,a
	ld hl,type_index
	jp find_pokemon_data_loop

is_already_captured:	; input : a=id
	ld hl,pokemon_stats
	ld b,(pokemon_stats_end-pokemon_stats)/(pokemon_stats_lenght-pokemon_stats)

is_already_captured_loop:
	ld c,(hl)
	cp c
	ret z
	ld d,0
	ld e,pokemon_stats_lenght-pokemon_stats
	add hl,de
	djnz is_already_captured_loop
	or $80
	ret

wait:	;	pause=480+300*b ns at 6MHz
	ei

waitloop:
	halt
	djnz wait
	di
	ret

cp_hl_de:
	or a
	sbc hl,de
	add hl,de
	ret

jp_hl:
	jp (hl)

_vputmap_onscreen:
	res textwrite,(IY+sgrflags)
	bcall _vputmap
	set textwrite,(IY+sgrflags)
	ret

dialogue:
	res textwrite,(IY+sgrflags)
	call _v_new_text
	jr _vputs_ml

_vdispa:
	ld h,0
	ld l,a
	bcall _setxxxxop2
	bcall _op2toop1
	ld a,3
	bcall _dispop1a
	ret

_vputs_ml:	;	_v_mode:1=text is typing; 0=text is directly drawn	|	string:$d6=new line; $07=pause and next text
	ld a,(pencol)
	push af
	push hl

_v_inc_loop:
	ld a,(hl)
	cp $d6
	jr z,_v_replace_zero
	cp 7
	jr z,_v_pause
	or a
	jr z,_v_disp_end
	inc hl
	jr _v_inc_loop

_v_replace_zero:
	ld (hl),0
	ex de,hl
	pop hl
	push de
_v_mode = $+1
	ld a,0
	or a
	bcallz _vputs
	call nz,_v_type
	pop de
	pop af
	push af
	push hl
	call _v_new_line
	ex de,hl
	ld (hl),$d6
	inc hl
	jr _v_inc_loop

_v_pause:
	ld (hl),0
	ex de,hl
	pop hl
	push de
	ld a,(_v_mode)
	or a
	bcallz _vputs
	call nz,_v_type
	push hl
	call _v_disp_pause
	pop hl
	pop de
	push hl
	xor a
	call _v_new_text
	ex de,hl
	ld (hl),7
	inc hl
	jr _v_inc_loop

_v_disp_pause:
	call lcd_to_gbuf
_v_disp_no_pause = $+1
	ld a,0
	or a
	ret nz
	call _v_disp_pause_blink

_v_disp_pause_loop:
_v_disp_pause_count = $+1
	ld a,0
	inc a
	ld (_v_disp_pause_count),a
	cp scroll_cursor_speed
	call z,_v_disp_pause_blink
	ld b,1
	call wait
	ld a,$bf
	out (1),a
	in a,(1)
	bit 5,a
	ret z
	jr _v_disp_pause_loop

_v_disp_pause_blink:
	xor a
	ld (_v_disp_pause_count),a
	ld a,86
	ld l,54
	ld b,8
	ld ix,scroll_cursor
	call ionPutSprite
	jp ionFastCopy

_v_disp_end:
	pop hl
	pop af
	ld a,(_v_mode)
	or a
	bcallz _vputs
	push af
	call nz,_v_type
	pop af
	call nz,_v_disp_pause
	set textwrite,(IY+sgrflags)
	ret

_v_new_line:
	ld (pencol),a
	ld a,(penrow)
	add a,6
	ld (penrow),a
	ret

_v_new_text:	;	a:0=frame on the bottom ; 1=frame on the top ; 2=battle frame
	push de
	push bc
	push hl
	push af
	ld a,2
	ld (pencol),a
	ld (penrow),a
	dec a
	ld (_v_mode),a
	pop af
	or a
	jr z,dialogue_down
	cp 1
	jr z,dialogue_up
	ld a,50
	ld (penrow),a
	ld de,$0031
	ld hl,$5f3f
	jr dialogue_end

dialogue_up
	ld de,0
	ld hl,$5f15
	jr dialogue_end

dialogue_down:
	ld a,44
	ld (penrow),a
	ld de,$002B
	ld hl,$5F3F

dialogue_end:
	call draw_frame	; bcall _DrawRectBorder not available on 83
	call ionFastCopy
	pop hl
	pop bc
	pop de
	ret

_v_type:
	ld a,$df
	out (1),a
	in a,(1)
	bit 7,a
	jr z,_v_type_skip

	ld a,(hl)
	or a
	ret z
	inc hl
	cp 7
	jr z,_v_type
	cp $d6
	jr z,_v_type

	push bc
	ld b,5
	call wait
	pop bc

	bcall _vputmap
	jr _v_type

_v_type_skip:
	bcall _vputs
	ret

draw_frame:
	ld c,1
	call rectangle_filled
	inc d
	inc e
	dec h
	dec l
	ld c,0
	jp rectangle_filled

lcd_to_gbuf:	; Screen grab routine by Ian Graf
	ld hl,gbuf
	di
	ld a,07h
	bcall _lcdbusy
	out (10h),a
	ld a,80h
	ld d,a

lcd_to_gbuf_loop:
	bcall _lcdbusy
	out (10h),a
	ld a,20h
	bcall _lcdbusy
	out (10h),a
	bcall _lcdbusy
	in a,(11h)
	ld bc,12*256+11h

lcd_to_gbuf_loop2:
	bcall _lcdbusy
	ini
	jr nz,lcd_to_gbuf_loop2
	inc d
	ld a,d
	cp 0C0h
	jr nz,lcd_to_gbuf_loop
	ld a,05h
	bcall _lcdbusy
	out (10h),a
	ei
	ret

fade_out:
	ld a,(contrast)
	ld b,a

fade_out_loop:
	push bc
	ld a,b
	ld (contrast),a
	add a,$18
	or $c0
	bcall _lcdbusy
	out (10h),a
	ld b,fade_speed
	call wait
	pop bc
	djnz fade_out_loop
	ret

fade_in:
	ld hl,contrast
contrast_var = $+1
	ld a,0
	sub (hl)
	ld b,a

fade_in_loop:
	push bc
	inc (hl)
	ld a,(hl)
	add a,$18
	or $c0
	bcall _lcdbusy
	out (10h),a
	ld b,fade_speed
	call wait
	pop bc
	djnz fade_in_loop
	ret

flash_screen:
	push bc
	ld de,0
	ld hl,$6040
	ld c,2
	call rectangle_filled
	call ionFastCopy
	ld de,0
	ld hl,$6040
	ld c,2
	call rectangle_filled
	call ionFastCopy
	pop bc
	djnz flash_screen
	ret

;=======================================;
; Rectangle Filling Routine Version 1.0 ;
; By Jason Kovacs & The TCPA - 10/11/99 ;
;=======================================;
; Input:  D = Top Left X Coordinate, E = Top Left Y Coordinate
;	 H = Bottom Right X Coord,  L = Bottom Right Y Coord
;	 C = Color of Lines (0-White, 1-Black, 2-XORed)
;
; Output: A Rectangle is drawn to the Graph Buffer with its border
;	 and everything within it Filled in according to the value in
;	 reg C which specifies the Color.
;
; Registers Affected: AF Destroyed; B=0 ; C, DE, HL Preserved.
;	 The Index Registers and the Shadow Registers Aren't Used.

rectangle_filled:
	ld a,l
	sub e
	inc a
	ld b,a
	ld a,h
	sub d
	inc a
	push de

rect_fill_loop:
	push af
	call V_Line
	pop af
	inc d
	dec a
	jr nz,rect_fill_loop
	pop de
	ret

;=======================================;
; Horizontal and Verticle Line Routines ;
; By Jason Kovacs & The TCPA - 10/11/99 ;
;=======================================;

; For H_Line and V_Line:
;
; Input:  B = Length of Line (Number of Pixels)
;	 C = Color of Line (0-White, 1-Black, 2-XORed)
;	 D = X Coordinate Start of the Line
;	 E = Y Coordinate Start of the Line
;
; Output: Lines are Drawn to the Graph Buffer, and the Starting
;	 Byte and Pixel Mask are Automatically determined according
;	 to the Input of the Coordinates in DE.
;
; Registers Affected:  All Registers are Preserved Except AF.

V_Line:
	push de
	push hl
	push bc
	ld a,d
	call Getpix
	pop bc
	push bc
	ld d,c
	ld c,a
	ld a,d
	ld de,12
	or a
	call z,V_White_Line
	dec a
	call z,V_Black_Line
	dec a
	call z,V_XORed_Line
	pop bc
	pop hl
	pop de
	ret

V_White_Line:
	ld a,c
	cpl
	ld c,a

V_White_Line_2:
	ld a,(hl)
	and c
	ld (hl),a
	add hl,de
	djnz V_White_Line_2
	xor a
	ret

V_Black_Line:
	ld a,(hl)
	or c
	ld (hl),a
	add hl,de
	djnz V_Black_Line
	xor a
	ret

V_XORed_Line:
	ld a,(hl)
	xor c
	ld (hl),a
	add hl,de
	djnz V_XORed_Line
	ret

Getpix:
	ld d,0
	ld h,d
	ld l,e
	add hl,de
	add hl,de
	add hl,hl
	add hl,hl
	ld de,plotsscreen
	add hl,de

Getbit:
	ld b,0
	ld c,a
	and %00000111
	srl c
	srl c
	srl c
	add hl,bc
	ld b,a
	inc b
	ld a,%00000001

GBLoop:
	rrca
	djnz GBLoop
	ret

; INPUT :
; ix:sprite; b:height; c:width/8
; OUTPUT :
; ix=flipped sprite
; destroys a, de
; uses saferam3

flip_sprite:
	push hl
	push bc
	push ix
	pop de
	ld hl,saferam3

flip_sprite_loop:
	push bc
	push de
	ld b,0
	add hl,bc
	pop de
	push hl
	ld b,c

flip_sprite_loop2:
	push bc
	dec hl
	ld a,(de)
	call reverse_a
	ld (hl),a
	inc de
	pop bc
	djnz flip_sprite_loop2
	pop hl
	pop bc
	djnz flip_sprite_loop
	ld hl,saferam3
	push hl
	pop ix
	pop bc
	pop hl
	ret

;- Reverse a
;input:	Byte in A
;output:	Reversed byte in A
;destroys	B
;Clock cycles: 66
;Bytes: 18
;author: calcmaniac84
reverse_a:
	ld b,a
	rrca
	rrca
	xor b
	and %10101010
	xor b
	ld b,a
	rrca
	rrca
	rrca
	rrca
	xor b
	and %01100110
	xor b
	rrca
	ret